#version 330

layout (location = 0) in vec3 pos;
layout (location = 1) in vec2 tex;
layout (location = 2) in vec3 norm;
layout (location = 3) in vec3 tangent;
layout (location = 4) in vec3 bitangent;

out vec4 vCol;
out vec2 TexCoord;
out mat3 TBN;
out vec3 FragPos;
out vec4 FragPosLightSpace;
out vec3 Normal;

uniform mat4 model;
uniform mat4 projection;
uniform mat4 view;
uniform float animateTime;
uniform mat4 lightSpaceMatrix;

uniform float k; 
uniform float velocity; 
uniform float amp;


void main()
{
	//
	vec4 pos1  = vec4(pos, 1.0f);
	float u = k*(pos1.y- velocity*animateTime);
	pos1.z = amp*sin(u);

	vec3 n = vec3(0.0);
	n.xy = normalize(vec2(-k*amp*cos(u), 1.0));
	
	//	vec4 pos1 = vec4( pos + sin(animateTime) * norm*200 , 1.0f);
	//vec4 pos1 = vec4(pos,1.0);
	
	gl_Position = projection * view * model * pos1;
	vCol = vec4(1.0f, 0.0f, 0.0f, 1.0f);
	
	TexCoord = tex;
	
	Normal = mat3(transpose(inverse(model))) * n;
	
	FragPos = (model * pos1).xyz; 

	
	FragPosLightSpace = lightSpaceMatrix * pos1; 

	
   vec3 T = normalize(vec3(model * vec4(tangent,   0.0)));
   vec3 B = normalize(vec3(model * vec4(bitangent, 0.0)));
   vec3 N = normalize(vec3(model * vec4(n,    0.0)));
   TBN = mat3(T, B, N);
}