#version 330

layout (location = 0) in vec3 pos;
layout (location = 1) in vec2 tex;
layout (location = 2) in vec3 norm;
layout (location = 3) in vec3 tangent;
layout (location = 4) in vec3 bitangent;

out vec4 vCol;
out vec2 TexCoord;
out mat3 TBN;
out vec3 FragPos;
out vec4 FragPosLightSpace;

uniform mat4 model;
uniform mat4 projection;
uniform mat4 view;
uniform mat4 lightSpaceMatrix;

void main()
{

	gl_Position = projection * view * model * vec4(pos, 1.0);
	vCol = vec4(clamp(pos, 0.0f, 1.0f), 1.0f);

	TexCoord = tex;
	
	FragPos = (model * vec4(pos, 1.0)).xyz; 
	
	FragPosLightSpace = lightSpaceMatrix * vec4(FragPos, 1.0); 

	mat3 invTransModel = mat3(transpose(inverse(model)));
	vec3 T = normalize(invTransModel * tangent);
	vec3 B = normalize(invTransModel * bitangent);
	vec3 N = normalize(invTransModel * norm);
	TBN = mat3(T, B, N);
}